/* $Id: cpl_test-test.c,v 1.15 2008/11/03 10:22:01 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2008/11/03 10:22:01 $
 * $Revision: 1.15 $
 * $Name: cpl-5_0_0 $
 */

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cpl_msg.h>
#include <cpl_type.h>
#include <cpl_test.h>

/*-----------------------------------------------------------------------------
                                  Defines
 -----------------------------------------------------------------------------*/

#ifndef IMAGESZ
#define IMAGESZ 100
#endif

/*-----------------------------------------------------------------------------
                                  Main
 -----------------------------------------------------------------------------*/
int main(void)
{

    int nxfail = 0; /* The number of expected failures */
    cpl_msg_severity level;
    cpl_image * img1;
    cpl_image * img2;
    cpl_polynomial * poly1;
    cpl_polynomial * poly2;
    cpl_mask       * mask1;
    cpl_mask       * mask2;

    cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);


    cpl_test(41);
    cpl_test_zero(0);

    cpl_test_null(NULL);
    cpl_test_nonnull(cpl_func);

    cpl_test_eq(1, 1);

    cpl_test_error(CPL_ERROR_NONE);

    /* Set a CPL error */
    (void)cpl_msg_set_log_name(NULL);
    cpl_test_error(CPL_ERROR_NULL_INPUT);

    cpl_test_noneq(0, 1);

    cpl_test_eq_string("", "");
    cpl_test_eq_string(cpl_func, cpl_func);

    cpl_test_abs(0.0, 0.0, 0.0);

    cpl_test_abs(0.0, 1.0, 1.0);
    cpl_test_abs(1.0, 0.0, 1.0);

    cpl_test_rel(0.0, 0.0, 0.0);

    cpl_test_rel(1.0, 1.0, 0.0);

    cpl_test_rel(-1.0,  -2.0, 1.0);
    cpl_test_rel(-2.0,  -1.0, 1.0);

    cpl_test_leq(0.0, 0.0);
    cpl_test_lt(0.0,  1.0);

    img1 = cpl_image_fill_test_create(IMAGESZ, IMAGESZ);
    img2 = cpl_image_duplicate(img1);

    cpl_test_image_abs(img1, img2, 0.0);

    cpl_image_add_scalar(img1, 1.0);

    cpl_test_image_abs(img1, img2, 1.0 + FLT_EPSILON); /* Round-off */

    poly1 = cpl_polynomial_new(2);
    poly2 = cpl_polynomial_duplicate(poly1);

    cpl_test_polynomial_abs(poly1, poly2, 0.0);

    mask1 = cpl_mask_new(IMAGESZ, IMAGESZ);
    mask2 = cpl_mask_new(IMAGESZ, IMAGESZ);
    cpl_mask_not(mask2);

    cpl_test_eq_mask(mask1, mask1);
    cpl_test_eq_mask(mask2, mask2);

    if (getenv(CPL_TEST_FITS)) {
        /* FIXME: Create real, valid FITS here ? */
    } else {
        cpl_test_fits("/dev/null");
    }

    /* Turn off messaging during testing of expected failures */

    level = cpl_msg_get_level();
    cpl_msg_set_level(CPL_MSG_OFF);

    cpl_test(0);                    nxfail++;
    cpl_test_zero(41);              nxfail++;

    cpl_test_null(cpl_func);        nxfail++;
    cpl_test_nonnull(NULL);         nxfail++;

    cpl_test_error(CPL_ERROR_EOL);  nxfail++;

    /* Set a CPL error */
    (void)cpl_msg_set_log_name(NULL);
    cpl_test_error(CPL_ERROR_NONE); nxfail++;

    cpl_test_eq(0, 1);              nxfail++;

    cpl_test_noneq(0, 0);           nxfail++;

    cpl_test_eq_string("A", "B");   nxfail++;
    cpl_test_eq_string(NULL, "");   nxfail++;
    cpl_test_eq_string(NULL, NULL); nxfail++;

    cpl_test_abs(0.0, 0.0, -1.0);   nxfail++;

    cpl_test_abs(0.0, 2.0, 1.0);    nxfail++;
    cpl_test_abs(2.0, 0.0, 1.0);    nxfail++;


    cpl_test_rel(0.0, 0.0, -1.0);   nxfail++;
    cpl_test_rel(0.0, 1.0, 1.0);    nxfail++;
    cpl_test_rel(1.0, 0.0, 1.0);    nxfail++;

    cpl_test_rel(1.0, 3.0, 1.0);    nxfail++;

    cpl_test_leq(1.0, 0.0);         nxfail++;
    cpl_test_lt(0.0,  0.0);         nxfail++;

    cpl_test_image_abs(NULL, NULL, 1.0); nxfail++;

    cpl_test_image_abs(img1, img2, 0.5); nxfail++;

    cpl_test_polynomial_abs(poly1, poly2, -1.0); nxfail++;

    cpl_polynomial_delete(poly2);
    poly2 = cpl_polynomial_new(cpl_polynomial_get_dimension(poly1)+1);

    cpl_test_polynomial_abs(poly1, poly2, 0.0); nxfail++;
    cpl_test_polynomial_abs(NULL,  poly2, 0.0); nxfail++;
    cpl_test_polynomial_abs(poly1, NULL,  0.0); nxfail++;
    cpl_test_polynomial_abs(NULL,  NULL,  0.0); nxfail++;

    cpl_test_image_abs(NULL, NULL, 1.0); nxfail++;

    cpl_test_eq_mask(mask1, NULL);  nxfail++;
    cpl_test_eq_mask(NULL,  mask1); nxfail++;
    cpl_test_eq_mask(mask1, mask2); nxfail++;

    cpl_mask_delete(mask2);
    mask2 = cpl_mask_new(IMAGESZ+1, IMAGESZ);
    cpl_test_eq_mask(mask1, mask2); nxfail++;

    cpl_mask_delete(mask2);
    mask2 = cpl_mask_new(IMAGESZ, IMAGESZ+1);
    cpl_test_eq_mask(mask1, mask2); nxfail++;

    cpl_test_fits(NULL); nxfail++;
    if (getenv(CPL_TEST_FITS)) {
        cpl_test_fits("/dev/null"); nxfail++;
    }

    /* Tests done - reinstate normal messaging */

    cpl_msg_set_level(level);

    cpl_msg_info(cpl_func, "Did %d failure tests", nxfail);

    cpl_image_delete(img1);
    cpl_image_delete(img2);
    cpl_polynomial_delete(poly1);
    cpl_polynomial_delete(poly2);
    cpl_mask_delete(mask1);
    cpl_mask_delete(mask2);

    return cpl_test_end(-nxfail);

}
